//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use core::ffi::*;
use core::ptr::NonNull;
use objc2_core_foundation::*;

use crate::*;

/// [Apple's documentation](https://developer.apple.com/documentation/coregraphics/kcgnotifyguiconsolesessionchanged?language=objc)
pub const kCGNotifyGUIConsoleSessionChanged: &CStr = unsafe {
    CStr::from_bytes_with_nul_unchecked(b"com.apple.coregraphics.GUIConsoleSessionChanged\0")
};
/// [Apple's documentation](https://developer.apple.com/documentation/coregraphics/kcgnotifyguisessionuserchanged?language=objc)
pub const kCGNotifyGUISessionUserChanged: &CStr = unsafe {
    CStr::from_bytes_with_nul_unchecked(b"com.apple.coregraphics.GUISessionUserChanged\0")
};
#[inline]
pub extern "C-unwind" fn CGSessionCopyCurrentDictionary() -> Option<CFRetained<CFDictionary>> {
    extern "C-unwind" {
        fn CGSessionCopyCurrentDictionary() -> Option<NonNull<CFDictionary>>;
    }
    let ret = unsafe { CGSessionCopyCurrentDictionary() };
    ret.map(|ret| unsafe { CFRetained::from_raw(ret) })
}
