//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use core::ffi::*;
#[cfg(feature = "objc2")]
use objc2::__framework_prelude::*;

use crate::*;

/// [Apple's documentation](https://developer.apple.com/documentation/coregraphics/cgdevicecolor?language=objc)
#[repr(C)]
#[derive(Clone, Copy, Debug, PartialEq)]
pub struct CGDeviceColor {
    pub red: c_float,
    pub green: c_float,
    pub blue: c_float,
}

#[cfg(feature = "objc2")]
unsafe impl Encode for CGDeviceColor {
    const ENCODING: Encoding = Encoding::Struct(
        "CGDeviceColor",
        &[
            <c_float>::ENCODING,
            <c_float>::ENCODING,
            <c_float>::ENCODING,
        ],
    );
}

#[cfg(feature = "objc2")]
unsafe impl RefEncode for CGDeviceColor {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}
