//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use core::ptr::NonNull;
#[cfg(feature = "objc2")]
use objc2::__framework_prelude::*;
use objc2_core_foundation::*;

use crate::*;

/// [Apple's documentation](https://developer.apple.com/documentation/coregraphics/cgtonemapping?language=objc)
// NS_ENUM
#[repr(transparent)]
#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub struct CGToneMapping(pub u32);
impl CGToneMapping {
    #[doc(alias = "kCGToneMappingDefault")]
    pub const Default: Self = Self(0);
    #[doc(alias = "kCGToneMappingImageSpecificLumaScaling")]
    pub const ImageSpecificLumaScaling: Self = Self(1);
    #[doc(alias = "kCGToneMappingReferenceWhiteBased")]
    pub const ReferenceWhiteBased: Self = Self(2);
    #[doc(alias = "kCGToneMappingITURecommended")]
    pub const ITURecommended: Self = Self(3);
    #[doc(alias = "kCGToneMappingEXRGamma")]
    pub const EXRGamma: Self = Self(4);
    #[doc(alias = "kCGToneMappingNone")]
    pub const None: Self = Self(5);
}

#[cfg(feature = "objc2")]
unsafe impl Encode for CGToneMapping {
    const ENCODING: Encoding = u32::ENCODING;
}

#[cfg(feature = "objc2")]
unsafe impl RefEncode for CGToneMapping {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

/// [Apple's documentation](https://developer.apple.com/documentation/coregraphics/cgcontenttonemappinginfo?language=objc)
#[repr(C)]
#[derive(Clone, Copy, Debug, PartialEq)]
pub struct CGContentToneMappingInfo {
    pub method: CGToneMapping,
    pub options: *const CFDictionary,
}

#[cfg(feature = "objc2")]
unsafe impl Encode for CGContentToneMappingInfo {
    const ENCODING: Encoding = Encoding::Struct(
        "?",
        &[<CGToneMapping>::ENCODING, <*const CFDictionary>::ENCODING],
    );
}

#[cfg(feature = "objc2")]
unsafe impl RefEncode for CGContentToneMappingInfo {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/coregraphics/kcgpreferreddynamicrange?language=objc)
    pub static kCGPreferredDynamicRange: &'static CFString;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/coregraphics/kcgdynamicrangehigh?language=objc)
    pub static kCGDynamicRangeHigh: &'static CFString;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/coregraphics/kcgdynamicrangeconstrained?language=objc)
    pub static kCGDynamicRangeConstrained: &'static CFString;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/coregraphics/kcgdynamicrangestandard?language=objc)
    pub static kCGDynamicRangeStandard: &'static CFString;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/coregraphics/kcgcontentaveragelightlevel?language=objc)
    pub static kCGContentAverageLightLevel: &'static CFString;
}

extern "C" {
    /// [Apple's documentation](https://developer.apple.com/documentation/coregraphics/kcgcontentaveragelightlevelnits?language=objc)
    pub static kCGContentAverageLightLevelNits: &'static CFString;
}
